@@ -8,8 +8,8 @@ class TaskAA(pyutilib.workflow.Task):
         pyutilib.workflow.Task.__init__(self, *args, **kwds)
         self.inputs.declare('x')
         self.inputs.declare('y')
-        self.parser.add_option('--x', dest='x', type='int')
-        self.parser.add_option('--y', dest='y', type='int')
+        self.parser.add_argument('--x', dest='x', type=int)
+        self.parser.add_argument('--y', dest='y', type=int)
         self.outputs.declare('z')
 
     def execute(self):
