@@ -18,8 +18,8 @@ from task import *
 from workflow import *
 from file import *
 from executable import *
-from options import *
 from tasks import *
+from driver import *
 
 pyutilib.component.core.PluginGlobals.pop_env()
 
