@@ -12,7 +12,7 @@
 __all__ = ['Task', 'EmptyTask', 'Component']
 
 import port
-from options import OptionParser
+import argparse
 import globals
 from pyutilib.misc import Options
 import pprint
@@ -23,7 +23,7 @@ class Task(object):
     A Task object represents a single action in a workflow.
     """
 
-    def __init__(self, id=None, name=None):
+    def __init__(self, id=None, name=None, parser=None):
         """Constructor."""
         if not id is None:
             self.id = id
@@ -39,7 +39,11 @@ class Task(object):
         self.outputs.set_name(self.name+"-outputs")
         self._resources = {}
         self._predecessors = []
-        self._create_parser()
+        self._create_parser(parser)
+        self.input_controls = port.InputPorts(self)
+        self.input_controls.set_name(self.name+'-input-controls')
+        self.output_controls = port.OutputPorts(self)
+        self.output_controls.set_name(self.name+'-output-controls')
 
     def add_resource(self, resource):
         """Add a resource that is required for this task to execute."""
@@ -51,11 +55,15 @@ class Task(object):
 
     def next_tasks(self):
         """Return the set of tasks that succeed this task in the workflow."""
-        return set(t.to_port.task for name in self.outputs for t in self.outputs[name].output_connections)
+        #print set(t.to_port.task for name in self.outputs for t in self.outputs[name].output_connections) 
+        #for name in self.output_controls:
+           #print name, self.output_controls[name]
+        #print set(t.to_port.task for name in self.output_controls for t in self.output_controls[name].output_connections)
+        return set(t.to_port.task for name in self.outputs for t in self.outputs[name].output_connections) | set(t.to_port.task for name in self.output_controls for t in self.output_controls[name].output_connections)
             
     def prev_tasks(self):
         """Return the set of tasks that precede this task in the workflow."""
-        return set([task for name in self.inputs for task in self.inputs[name].from_tasks() if task.id != NoTask.id]) | set(task for task in self._predecessors)
+        return set([task for name in self.inputs for task in self.inputs[name].from_tasks() if task.id != NoTask.id]) | set(task for task in self._predecessors) | set([task for name in self.input_controls for task in self.input_controls[name].from_tasks() if task.id != NoTask.id])
            
     def next_task_ids(self):
         """Return the set of ids for tasks that succeed this task in the workflow."""
@@ -69,10 +77,31 @@ class Task(object):
         """Execute this task."""
         raise ValueError, "There is no default execution for an abstract Task object! Task=%s" % str(self) #pragma:nocover
 
-    def busy(self):
+    def busy_resources(self):
         """Return the list of resources that this task is waiting for."""
         return [name for name in self._resources if not self._resources[name].available()]
 
+    def ready(self):
+        if self.busy():
+            return False
+        for name in self.inputs:
+            #print "XYZ",self.name, name, self.inputs[name].ready(),self.inputs[name]._ready
+            #for connection in self.inputs[name].input_connections:
+                #print "XYZ",self.name, name,connection.from_port._ready, connection.ready(), len(connection.from_port.input_connections), connection.from_port.task.name
+            if not self.inputs[name].ready():
+                #print "FALSE - input", name
+                #print self.inputs[name]
+                return False
+        for name in self.input_controls:
+            if not self.input_controls[name].ready():
+                #print "FALSE - control", name
+                return False
+        return True
+
+    def busy(self):
+        """Return the list of resources that this task is waiting for."""
+        return len(self.busy_resources())
+
     def __call__(self, *options, **kwds):
         """Setup inputs and output parameters and execute this task.
 
@@ -85,7 +114,7 @@ class Task(object):
 
     def _call_init(self, *options, **kwds):
         self._call_start()
-        busy = self.busy()
+        busy = self.busy_resources()
         if len(busy) > 0:
             raise IOError, "Cannot execute task %s.  Busy resources: %s" % (self.name, str(busy))
         # Set inputs
@@ -99,7 +128,7 @@ class Task(object):
             #print "z",i,getattr(self.outputs,i).get_value()
             setattr(self, i, None)
         for i in self.inputs:
-            #print "X",i,self.inputs[i].get_value(),str(self.inputs[i])
+            #print "OIUOX",i,self.inputs[i].get_value(),str(self.inputs[i])
             # TODO: validate that non-optional inputs have a value other than None
             self.inputs[i].compute_value()
             setattr(self, i, self.inputs[i].get_value())
@@ -109,9 +138,11 @@ class Task(object):
             #print "Z",i,getattr(self.outputs,i).get_value()
             # TODO: validate that non-optional outputs have a value other than None
             self.outputs[i].set_value( getattr(self, i) )
+
         for name, res in self._resources.iteritems():
             res.unlock()
         self._call_finish()
+        self.set_ready( )
         #
         opt = Options()
         for i in self.outputs:
@@ -120,12 +151,12 @@ class Task(object):
 
     def set_options(self, args):
         """Use a list of command-line options to initialize this task."""
-        [self.options, args] = self.parser.parse_args(args)
+        [self.options, args] = self.parser.parse_known_args(args)
         tmp = {}
-        for opt in self.parser._long_opt:
+        for action in self.parser._actions:
             try:
-                val = getattr(self.options,opt[2:])
-                tmp[opt[2:]] = val
+                val = getattr(self.options, action.dest)
+                tmp[action.dest] = val
             except:
                 pass
         self._set_inputs(tmp)
@@ -143,14 +174,16 @@ class Task(object):
         for key in options:
             self.inputs[key].set_value(options[key])
 
-    def _create_parser(self):
+    def _create_parser(self, parser=None):
         """Create the OptionParser object and populated it with option groups."""
-        self.parser = OptionParser()
+        if parser is None:
+            self.parser = argparse.ArgumentParser()
+        else:
+            self.parser = parser
         self._parser_group = {}
         self._create_parser_groups()
-        self.parser.ignore_invalid()
         for key in self._parser_group:
-            self.parser.add_option_group(self._parser_group[key])
+            self.parser.add_argument_group(self._parser_group[key])
 
     def _create_parser_groups(self):
         """This method is called by the _create_parser method to setup the
@@ -163,7 +196,9 @@ class Task(object):
         tmp['Id'] = self.id
         tmp['Inputs'] = self.inputs._repn_()
         tmp['Outputs'] = self.outputs._repn_()
-        return tmp
+        tmp['InputControls'] = self.input_controls._repn_()
+        tmp['OutputControls'] = self.output_controls._repn_()
+        return tmp 
 
     def __repr__(self):
         """Return a string representation for this task."""
@@ -173,6 +208,18 @@ class Task(object):
         """Return a string representation for this task."""
         return "%s prev: %s next: %s resources: %s" % (str(self.name),str(sorted(list(self.prev_task_ids()))),str(sorted(list(self.next_task_ids()))), str(sorted(self._resources.keys())))
 
+    def reset(self):
+        #print "RESETING "+self.name
+        for i in self.outputs:
+            self.outputs[i].reset()
+        for i in self.output_controls:
+            self.output_controls[i].reset()
+
+    def set_ready(self):
+        for i in self.outputs:
+            self.outputs[i].set_ready()
+        
+
 
 class Component(Task):
     """
