@@ -75,10 +75,10 @@ class TaskBB(TaskB):
     def __init__(self, *args, **kwds):
         TaskB.__init__(self, *args, **kwds)
 
-    def _create_parser(self):
-        pyutilib.workflow.Task._create_parser(self)
-        self.parser.add_option("--a",dest="a", type='int')
-        self.parser.add_option("--i",dest="i", type='int')
+    def _create_parser(self, parser=None):
+        pyutilib.workflow.Task._create_parser(self, parser)
+        self.parser.add_argument("--a",dest="a", type=int)
+        self.parser.add_argument("--i",dest="i", type=int)
 
 
 class TaskC(pyutilib.workflow.Task):
@@ -97,9 +97,9 @@ class TaskCC(TaskC):
     def __init__(self, *args, **kwds):
         TaskC.__init__(self, *args, **kwds)
 
-    def _create_parser(self):
-        pyutilib.workflow.Task._create_parser(self)
-        self.parser.add_option("--i",dest="i", type='int')
+    def _create_parser(self, parser=None):
+        pyutilib.workflow.Task._create_parser(self, parser)
+        self.parser.add_argument("--i",dest="i", type=int)
 
 
 class TaskD(pyutilib.workflow.Task):
@@ -146,7 +146,7 @@ class Test(unittest.TestCase):
         #
         ##print "..."
         self.failUnlessEqual(str(w),\
-"""Workflow:
+"""Workflow Task4:
 Task5 prev: [] next: [2, 3] resources: []
 Task2 prev: [5] next: [1] resources: []
 Task3 prev: [5] next: [1] resources: []
@@ -174,7 +174,7 @@ Task6 prev: [1] next: [] resources: []""")
         #
         ##print "..."
         self.failUnlessEqual(str(w),\
-"""Workflow:
+"""Workflow Task5:
 Task6 prev: [] next: [4] resources: []
 Task4 prev: [6] next: [2, 3] resources: []
 Task2 prev: [4] next: [1] resources: []
@@ -195,15 +195,15 @@ Task7 prev: [1] next: [] resources: []""")
         # Create workflow
         w = pyutilib.workflow.Workflow()
         w.add(A,loadall=False)
-        self.failUnlessEqual(str(w),"Workflow:\nTask5 prev: [] next: [] resources: []")
+        self.failUnlessEqual(str(w),"Workflow Task4:\nTask5 prev: [] next: [] resources: []")
 
     def test1a(self):
         pyutilib.workflow.reset_id_counter()
         # Create workflow
         w = pyutilib.workflow.Workflow()
-        self.failUnlessEqual(str(w),"Workflow:\nTask2 prev: [] next: [] resources: []")
+        self.failUnlessEqual(str(w),"Workflow Task1:\nTask2 prev: [] next: [] resources: []")
         w.add(pyutilib.workflow.task.NoTask)
-        self.failUnlessEqual(str(w),"Workflow:\nTask2 prev: [] next: [] resources: []")
+        self.failUnlessEqual(str(w),"Workflow Task1:\nTask2 prev: [] next: [] resources: []")
 
     @unittest.skipIf( is_python24or25, "There is a slight (space) formatting difference from pformat from Python2.6.  Skipping test.")
     def test2(self):
@@ -216,32 +216,46 @@ Task7 prev: [1] next: [] resources: []""")
         A.inputs.x = B.outputs.b
         base = """{ 'A_TYPE': 'Task',
   'Id': 1,
+  'InputControls': { 'A_TYPE': 'Port',
+                     'Mode': 'inputs',
+                     'Name': 'A-input-controls',
+                     'Owner': 'A prev: [2] next: [] resources: []'},
   'Inputs': { 'A_TYPE': 'Port',
               'Mode': 'inputs',
               'Name': 'A-inputs',
               'Owner': 'A prev: [2] next: [] resources: []',
               'x': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1)'],
+                     'Action': 'store',
+                     'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1) False'],
                                       'Outputs': []},
                      'Name': 'x',
                      'Optional': 'False',
+                     'Ready': 'False',
                      'Task': '1',
                      'Value': 'None'},
               'y': { 'A_TYPE': 'Port',
+                     'Action': 'store',
                      'Connections': { 'Inputs': [], 'Outputs': []},
                      'Name': 'y',
                      'Optional': 'False',
+                     'Ready': 'False',
                      'Task': '1',
                      'Value': 'None'}},
   'Name': 'A',
+  'OutputControls': { 'A_TYPE': 'Port',
+                      'Mode': 'outputs',
+                      'Name': 'A-output-controls',
+                      'Owner': 'A prev: [2] next: [] resources: []'},
   'Outputs': { 'A_TYPE': 'Port',
                'Mode': 'outputs',
                'Name': 'A-outputs',
                'Owner': 'A prev: [2] next: [] resources: []',
                'z': { 'A_TYPE': 'Port',
+                      'Action': 'store',
                       'Connections': { 'Inputs': [], 'Outputs': []},
                       'Name': 'z',
                       'Optional': 'False',
+                      'Ready': 'False',
                       'Task': '1',
                       'Value': 'None'}}}"""
         self.failUnlessEqual(repr(A), base)
@@ -260,16 +274,20 @@ Task7 prev: [1] next: [] resources: []""")
   'Name': 'Task1-inputs',
   'Owner': 'Task1 prev: [2] next: [] resources: []',
   'x': { 'A_TYPE': 'Port',
-         'Connections': { 'Inputs': ['DirectConnector: from=(2) to=(1)'],
+         'Action': 'store',
+         'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1) False'],
                           'Outputs': []},
          'Name': 'x',
          'Optional': 'False',
+         'Ready': 'False',
          'Task': '1',
          'Value': 'None'},
   'y': { 'A_TYPE': 'Port',
+         'Action': 'store',
          'Connections': { 'Inputs': [], 'Outputs': []},
          'Name': 'y',
          'Optional': 'False',
+         'Ready': 'False',
          'Task': '1',
          'Value': 'None'}}"""
 
@@ -278,9 +296,11 @@ Task7 prev: [1] next: [] resources: []""")
   'Name': 'Task1-outputs',
   'Owner': 'Task1 prev: [2] next: [] resources: []',
   'z': { 'A_TYPE': 'Port',
+         'Action': 'store',
          'Connections': { 'Inputs': [], 'Outputs': []},
          'Name': 'z',
          'Optional': 'False',
+         'Ready': 'False',
          'Task': '1',
          'Value': 'None'}}"""
 
@@ -298,31 +318,45 @@ Task7 prev: [1] next: [] resources: []""")
         A.inputs['x'] = 2
         base = """{ 'A_TYPE': 'Task',
   'Id': 1,
+  'InputControls': { 'A_TYPE': 'Port',
+                     'Mode': 'inputs',
+                     'Name': 'Task1-input-controls',
+                     'Owner': 'Task1 prev: [] next: [] resources: []'},
   'Inputs': { 'A_TYPE': 'Port',
               'Mode': 'inputs',
               'Name': 'Task1-inputs',
               'Owner': 'Task1 prev: [] next: [] resources: []',
               'x': { 'A_TYPE': 'Port',
+                     'Action': 'store',
                      'Connections': { 'Inputs': [], 'Outputs': []},
                      'Name': 'x',
                      'Optional': 'False',
+                     'Ready': 'True',
                      'Task': '1',
                      'Value': '2'},
               'y': { 'A_TYPE': 'Port',
+                     'Action': 'store',
                      'Connections': { 'Inputs': [], 'Outputs': []},
                      'Name': 'y',
                      'Optional': 'False',
+                     'Ready': 'False',
                      'Task': '1',
                      'Value': 'None'}},
   'Name': 'Task1',
+  'OutputControls': { 'A_TYPE': 'Port',
+                      'Mode': 'outputs',
+                      'Name': 'Task1-output-controls',
+                      'Owner': 'Task1 prev: [] next: [] resources: []'},
   'Outputs': { 'A_TYPE': 'Port',
                'Mode': 'outputs',
                'Name': 'Task1-outputs',
                'Owner': 'Task1 prev: [] next: [] resources: []',
                'z': { 'A_TYPE': 'Port',
+                      'Action': 'store',
                       'Connections': { 'Inputs': [], 'Outputs': []},
                       'Name': 'z',
                       'Optional': 'False',
+                      'Ready': 'False',
                       'Task': '1',
                       'Value': 'None'}}}"""
 
@@ -385,152 +419,10 @@ Task7 prev: [1] next: [] resources: []""")
         w.add(A)
         w.set_options(['--i=3','--a=2'])
         w()
-        self.assertEqual(repr(w),"""Workflow:
-{ 'A_TYPE': 'Task',
-  'Id': 5,
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task5-inputs',
-              'Owner': 'Task5 prev: [] next: [2, 3] resources: []'},
-  'Name': 'Task5',
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task5-outputs',
-               'Owner': 'Task5 prev: [] next: [2, 3] resources: []',
-               'a': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [],
-                                       'Outputs': [ 'DirectConnector: from=(5) to=(2)']},
-                      'Name': 'a',
-                      'Optional': 'False',
-                      'Task': '5',
-                      'Value': 'None'},
-               'i': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [],
-                                       'Outputs': [ 'DirectConnector: from=(5) to=(3)',
-                                                    'DirectConnector: from=(5) to=(2)']},
-                      'Name': 'i',
-                      'Optional': 'False',
-                      'Task': '5',
-                      'Value': 'None'},
-               'x': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [], 'Outputs': []},
-                      'Name': 'x',
-                      'Optional': 'False',
-                      'Task': '5',
-                      'Value': 'None'},
-               'y': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [], 'Outputs': []},
-                      'Name': 'y',
-                      'Optional': 'False',
-                      'Task': '5',
-                      'Value': 'None'}}}
-{ 'A_TYPE': 'Task',
-  'Id': 2,
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task2-inputs',
-              'Owner': 'Task2 prev: [5] next: [1] resources: []',
-              'a': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(5) to=(2)'],
-                                      'Outputs': []},
-                     'Name': 'a',
-                     'Optional': 'False',
-                     'Task': '2',
-                     'Value': '2'},
-              'i': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(5) to=(2)'],
-                                      'Outputs': []},
-                     'Name': 'i',
-                     'Optional': 'False',
-                     'Task': '2',
-                     'Value': '3'}},
-  'Name': 'Task2',
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task2-outputs',
-               'Owner': 'Task2 prev: [5] next: [1] resources: []',
-               'b': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [],
-                                       'Outputs': [ 'DirectConnector: from=(2) to=(1)']},
-                      'Name': 'b',
-                      'Optional': 'False',
-                      'Task': '2',
-                      'Value': '304'}}}
-{ 'A_TYPE': 'Task',
-  'Id': 3,
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task3-inputs',
-              'Owner': 'Task3 prev: [5] next: [1] resources: []',
-              'i': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(5) to=(3)'],
-                                      'Outputs': []},
-                     'Name': 'i',
-                     'Optional': 'False',
-                     'Task': '3',
-                     'Value': '3'}},
-  'Name': 'Task3',
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task3-outputs',
-               'Owner': 'Task3 prev: [5] next: [1] resources: []',
-               'o': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [],
-                                       'Outputs': [ 'DirectConnector: from=(3) to=(1)']},
-                      'Name': 'o',
-                      'Optional': 'False',
-                      'Task': '3',
-                      'Value': '30'}}}
-{ 'A_TYPE': 'Task',
-  'Id': 1,
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task1-inputs',
-              'Owner': 'Task1 prev: [2, 3] next: [6] resources: []',
-              'x': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1)'],
-                                      'Outputs': []},
-                     'Name': 'x',
-                     'Optional': 'False',
-                     'Task': '1',
-                     'Value': '304'},
-              'y': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(3) to=(1)'],
-                                      'Outputs': []},
-                     'Name': 'y',
-                     'Optional': 'False',
-                     'Task': '1',
-                     'Value': '30'}},
-  'Name': 'Task1',
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task1-outputs',
-               'Owner': 'Task1 prev: [2, 3] next: [6] resources: []',
-               'z': { 'A_TYPE': 'Port',
-                      'Connections': { 'Inputs': [],
-                                       'Outputs': [ 'DirectConnector: from=(1) to=(6)']},
-                      'Name': 'z',
-                      'Optional': 'False',
-                      'Task': '1',
-                      'Value': '334'}}}
-{ 'A_TYPE': 'Task',
-  'Id': 6,
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task6-inputs',
-              'Owner': 'Task6 prev: [1] next: [] resources: []',
-              'z': { 'A_TYPE': 'Port',
-                     'Connections': { 'Inputs': [ 'DirectConnector: from=(1) to=(6)'],
-                                      'Outputs': []},
-                     'Name': 'z',
-                     'Optional': 'False',
-                     'Task': '6',
-                     'Value': '334'}},
-  'Name': 'Task6',
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task6-outputs',
-               'Owner': 'Task6 prev: [1] next: [] resources: []'}}""")
+        OUTPUT = open(currdir+'test5b.out','w')
+        print >>OUTPUT, repr(w)
+        OUTPUT.close()
+        self.failUnlessFileEqualsBaseline(currdir+'test5b.out', currdir+'test5b.txt')
 
     def test_error1(self):
         pyutilib.workflow.reset_id_counter()
@@ -604,6 +496,32 @@ Task7 prev: [1] next: [] resources: []""")
         except ValueError:
             pass
 
+    def test_error6(self):
+        driver = pyutilib.workflow.TaskDriver()
+        try:
+            driver.register_task('_foo_')
+            self.fail("Expected ValueError because '_foo_' task is not defined")
+        except ValueError:
+            pass
+
+    def test_driver1(self):
+        driver = pyutilib.workflow.TaskDriver()
+        driver.register_task('workflow.selection')
+        try:
+            driver.parse_args(['-h'])
+            self.fail("Expected system exit because of bad options")
+        except SystemExit:
+            pass
+        try:
+            driver.parse_args(['foo'])
+            self.fail("Expected error because bad subcommand is specified")
+        except SystemExit:
+            pass
+        try:
+            driver.parse_args(['workflow.selection'])
+            self.fail("Expected error because task input is not defined")
+        except ValueError:
+            pass
 
 if __name__ == "__main__":
     unittest.main()
